% *************************************************************************
% 
%                      iGem Modelling - 2016
%                     Delft University of Technology
% 
% *************************************************************************
% 
%               Title
% 
%                    File: solving_1pulse
%                           Model:The dynamics of GFP excitation for a pulsed laser determine minimal cell size 
% 
%             
% *************************************************************************
%
function [C,maxima_photons,tt,y]=Laser_threshold_sizescan
global GFPt QY ii Kstim Ksp Kabs Kmirror;
%%
% This function simulates the amount of excited GFP molecules in a lasing 
% system directly after a pulse assuming all the fluorophores are in the 
% excited state. 

%% Nature constants:
Na          = 6.022e23;                 % [N/mol]           Avogadro Number 
c           = 2.99792458e8;             % [m/s]             speed of light 

%% system size
    % System size for a Single cell
    n           = 100;                   %                  Number of points 
    l           = logspace(-7,-3,n);     % [m]              Cell diameter, assuming spherical  (also characteristic path length)
    V           = 4/3*pi.*(l/2).^3;      % [m^3]            Assuming spherical cell
    savename = ['Laser_threshold_Singlecell_dia_R02711_1016_sizescan'];

%% Characteristics of fluorophore (GFP):
t_sp        = 2.71e-9;                 % [s]               Lifetime GFP 
sigma_a     = 2.03e-20;                % [m^2/N]           GFP absorption cross section = Stimulated emission cross section 
QY          = 0.6;                     %                   Quantum yield
                    
%% System characteristics
C           = 20e-3;                    % [Molair]          Molaire concentration of GFP
GFPt        = C*10^3*V*Na;              % [N]               Total number of GFP molecules in the system

%% Rates
R           = 0.2711;                   %                   Reflectivity mirror              
Ksp         = 1 / t_sp;                 % [s^-1]            Spontaneous relaxation rate
alpha       = 15.2;                     % [m^-1]            Absorption coefficient cells
Kabs        = -alpha * c;               % [s^-1]            Absorption rate

%% Solving the differential equation
options1        = odeset('InitialStep',1e-15);
% ty              = cell(n,2)     
maxima_photons  = zeros(length(V),2);
%%
tf              = 1.5e-9;               % [s]               Stop time
tspan           = [0, tf];              % [s]               Time interval to solve ODE
for ii = 1:length(V)
   ii
   Kmirror     = c / (2 * l(ii)) * log((R)^2); % [s^-1]            Mirror loss rate
   Kstim       = sigma_a * c / V(ii);          % [s^-1 N^-1]       Stimulated emission rate
   [tt,y]               = ode45(@dp1,tspan,[GFPt(ii),0],options1);
%    ty{ii,1}             = sparse(tt);
%    ty{ii,2}             = sparse(y); 
   [maxy,i]             = max(y(:,2));
   maxima_photons(ii,:) = [maxy,tt(i)];
end
%% Plotting 
figure(1)
hold on 
plot(C*10^6,-maxima_photons(:,1)*Kmirror,'-o')
title('Maximal output intensity')
xlabel('Concentration [\muM]')
ylabel('number of photons [N/s]')

%% Saving variables
save(savename,'-v7.3')

end
%%
function dydt = dp1(t,y)
global GFPt QY ii Kstim Ksp Kabs Kmirror; 
dydt=zeros(2,1);
% % the differential equations
% Spontaneous +Stimulated +mirrors
% Number of GFP in excited state
dydt(1)             = -(Ksp + Kstim * y(2)) * y(1);                                 % dGFP1/dt 
if y(1)>GFPt(ii)
    % GFP1 is not aloud to be larger than the total number of GFP molecules
    y(1)            = GFPt(ii);
end
% Number of photons N1
dydt(2)             = (QY * Ksp + Kstim * y(2)) * y(1) + (Kmirror + Kabs) * y(2);   % dN1/dt
end
