function [R] = Reflection_thinfilm(n0,n1,n2,lambda,d,thetai1)
%% 
% This function determines the reflectivity from maxwell equations.
%% Input Variables
%           n0:         Refractive index of cytoplasm
%           n1:         Refractive index of thin film
%           n2:         Refractive index of buffer
%           lambda:     Wavelength of light
%           thetai1:    Angle of incidence
%% Output Variable
%           R: Reflectivity from input variables
%%
eps0        = 8.8541878176*10^-12;  % Vacuum permittivity
mu0         = 4*pi*10^-7;           % Vacuum permeability
k0          = 2*pi/lambda;          % Wavenumber

% Determining the outgomming angles
thetai2     = asin(n0/n1*sin(thetai1)); 
theta2      = asin(n0/n2*sin(thetai1)); 
% Phase shift
h           = (2*n1*d*cos(thetai2))/2; 

Y1          = (eps0/mu0)^(1/2)*n1*cos(thetai2); % Acounting for S-polarized light
Y1p         = (eps0/mu0)^(1/2)*n1/cos(thetai2); % Acoutning for P-polarized light
Y0          = (eps0/mu0)^(1/2)*n0*cos(thetai1); 
Ys          = (eps0/mu0)^(1/2)*n2*cos(theta2);

% Characteristic matrix elements
m11         = cos(k0*h);
m12         = i*sin(k0*h)/Y1;   % S-polarized light
m21         = Y1*i*sin(k0*h);   % S-Polarized light
m12p        = i*sin(k0*h)/Y1p;  % P-Polarized light
m21p        = Y1p*i*sin(k0*h);  % P-Polarized light
m22         = cos(k0*h);

% Determining the amplitude coefficient of reflection
r           = (Y0*m11+Y0*Ys*m12-m21-Ys*m22)/(Y0*m11+Y0*Ys*m12+m21+Ys*m22);      % For S-polarized light 
rp          = (Y0*m11+Y0*Ys*m12p-m21p-Ys*m22)/(Y0*m11+Y0*Ys*m12p+m21p+Ys*m22);  % For P-polarized light

% Determine Reflectivity 
Rs          = abs(r)^2;         % Reflectivity S-Polarized light
Rp          = abs(rp)^2;        % Reflectivity P-Polarized light
R           = 1/2*(Rs+Rp);      % Reflectivity unpolarized light
end
